#!/bin/bash

###################################################
#
# Name: mog-cloudinit.sh
# Author: Reggie Borik, reborik@cisco.com
#
####################################################
#
#
# Version: 0.1
# Date: 12/9/15
# Summary: Config-Drive script for MOG
#
# Version: 0.2
# Date: 12/16/15
# Summary: Debug commands added, interface activation
#   commands separated. Routine added to check eth0
#   and if found down, activate (again?!). Default
#   route commands fixed.
#
# Version: 0.3
# Date: 12/17/15
# Summary:  Replaced interface file writing to
#   using "echo" instead of cat << EOF >. Added
#   input file correction (extra space elimination)
#   and hostname match checking (no correction but
#   logging the number of matches).
#
# Version: 0.4
# Date: 1/4/16
# Summary: Added routine to delete dummy interfaces
#   in order to keep interface numbering consistent
#   with NDR/design.
#
# Version: 0.5
# Date: 2/11/16
# Summary: Added UTC configuration to all hosts.
#   Add route interface file creation with input
#   from vmconfig input.
#
# Version: 0.6.1
# Date: 3/11/16
# Summary: Mechanism to allow for delayed volume 
#   mounting on CM (AIC EMC serial threading 
#   workaround).
#
# Version: 0.6.2
# Date: 3/15/16
# Summary: 9.0 updates 
#   UTC Fix
#   
# Version: 0.6.5
# Date: 3/28/16
# Summary: 9.0 update
#   Added Arbiter functionality: 6th column in CSV
#   to insert lookup entries into /etc/hosts:
#   ETH0 ONLY!
#   Remove any newlines in vmconfig.csv
#
# Version: 0.6.7
# Date: 5/2/16
#   Summary:
#   Allow for VIP nets to be extracted
#
# Version: 0.7
#   Date: 5/12/16
#   Summary:
#   Arbiter Default Gateway Fix
#
#
# Version: 0.7.1
#   Date: 5/18/16
#   Summary: Add Linux utils on CM, 
#   OAM secondary route table on PD's, redo "hosts"
#   field for VIP IPs (oam routing), remove tr
#   -s "squeeze" as alters hostname with repeated
#   alpha characters
#
####################################################

logfile="/root/cloudinit-log.out"
exec > $logfile 2>&1
set -x

#TIMESTAMP LOG
echo `date`

mhost=`hostname`
host=`echo $mhost | tr  [:lower:] [:upper:]`
filename="/root/vmconfig.csv"

#  
until [[ -e $filename ]]
do
  sleep 1
done

#REMOVE ANY MS NEWLINES
sed -i -e 's'"/`echo \\\r`/"/g $filename


## READING IPS ##
match=0
for line in `cat $filename | sed s/" "//g`
do
  IFS=',' read -r -a record <<< "$line"
  if [[ ${record[0]} == \#* ]]
  then
    continue
  fi
  
  if [[ ${record[0]} == VIPNETS ]]	
  then
    vipnets=${record[1]}
    continue	
  fi

  tmp="${record[0]}_${record[1]}_ip"
  declare $tmp=${record[2]}
  tmp="${record[0]}_${record[1]}_gw"
  declare $tmp=${record[3]}
  tmp="${record[0]}_${record[1]}_mask"
  declare $tmp=${record[4]}
  tmp="${record[0]}_${record[1]}_routes"
  declare $tmp=${record[5]}
  #tmp="${record[0]}_${record[1]}_hosts"
  #declare $tmp=${record[6]}
  tmp="${record[0]}_${record[1]}_vipip"
  declare $tmp=${record[6]}
  if [[ "${record[0]}" == *CM* && "${record[1]}" == eth0 ]]
  then
    installer="${record[2]}"
  fi
  
  # Default GW Arbiter Fix
  if [[ "${record[0]}" == "$host" && "${record[1]}" == eth1 ]]
  then 
	default_oam_gw="${record[3]}"
  fi
 
  if [[ "${record[0]}" == "$host" && "${record[1]}" == eth2 ]]
  then 
	default_repl_gw="${record[3]}"
  fi
  
  if [[ "${record[0]}" == "$host" && "${record[1]}" == eth7 ]]
  then 
	default_inet_gw="${record[3]}"
  fi
  
  if [[ "${record[0]}" == "$host" ]]
  then
    match=$((match += 1))
  fi
done

echo "FOUND $match MATCHES OF HOSTNAME WITH INPUT FILE ENTRIES."
echo "THIS SHOULD EQUAL THE NUMBER OF EXPECTED CONFIGURED INTERFACES. IF NOT CHECK INPUT FILE."

if [[ $match == 0 ]]
then
  echo "NO HOSTNAME MATCHES FOUND. PROBABLE SPELLING OR CASE ERRORS. CHECK INPUT FILE."
fi

## DECLARING INTERFACE VARIABLES
eth0_ip=${host}_eth0_ip
echo ${!eth0_ip}
eth0_gw=${host}_eth0_gw
echo ${!eth0_gw}
eth0_mask=${host}_eth0_mask
echo ${!eth0_mask}
eth0_routes=${host}_eth0_routes
echo ${!eth0_routes}
eth0_vipip=${host}_eth0_vipip
echo ${!eth0_vipip}

eth1_ip=${host}_eth1_ip
echo ${!eth1_ip}
eth1_gw=${host}_eth1_gw
echo ${!eth1_gw}
eth1_mask=${host}_eth1_mask
echo ${!eth1_mask}
eth1_routes=${host}_eth1_routes
echo ${!eth1_routes}
eth1_vipip=${host}_eth1_vipip
echo ${!eth1_vipip}

eth2_ip=${host}_eth2_ip
echo ${!eth2_ip}
eth2_gw=${host}_eth2_gw
echo ${!eth2_gw}
eth2_mask=${host}_eth2_mask
echo ${!eth2_mask}
eth2_routes=${host}_eth2_routes
echo ${!eth2_routes}
eth2_vipip=${host}_eth2_vipip
echo ${!eth2_vipip}

eth3_ip=${host}_eth3_ip
echo ${!eth3_ip}
eth3_gw=${host}_eth3_gw
echo ${!eth3_gw}
eth3_mask=${host}_eth3_mask
echo ${!eth3_mask}
eth3_routes=${host}_eth3_routes
echo ${!eth3_routes}
eth3_vipip=${host}_eth3_vipip
echo ${!eth3_vipip}

eth4_ip=${host}_eth4_ip
echo ${!eth4_ip}
eth4_gw=${host}_eth4_gw
echo ${!eth4_gw}
eth4_mask=${host}_eth4_mask
echo ${!eth4_mask}
eth4_routes=${host}_eth4_routes
echo ${!eth4_routes}
eth4_vipip=${host}_eth4_vipip
echo ${!eth4_vipip}

eth5_ip=${host}_eth5_ip
echo ${!eth5_ip}
eth5_gw=${host}_eth5_gw
echo ${!eth5_gw}
eth5_mask=${host}_eth5_mask
echo ${!eth5_mask}
eth5_routes=${host}_eth5_routes
echo ${!eth5_routes}
eth5_vipip=${host}_eth5_vipip
echo ${!eth5_vipip}

eth6_ip=${host}_eth6_ip
echo ${!eth6_ip}
eth6_gw=${host}_eth6_gw
echo ${!eth6_gw}
eth6_mask=${host}_eth6_mask
echo ${!eth6_mask}
eth6_routes=${host}_eth6_routes
echo ${!eth6_routes}
eth6_vipip=${host}_eth6_vipip
echo ${!eth6_vipip}

eth7_ip=${host}_eth7_ip
echo ${!eth7_ip}
eth7_gw=${host}_eth7_gw
echo ${!eth7_gw}
eth7_mask=${host}_eth7_mask
echo ${!eth7_mask}
eth7_routes=${host}_eth7_routes
echo ${!eth7_mask}
eth7_vipip=${host}_eth7_vipip
echo ${!eth7_vipip}

eth8_ip=${host}_eth8_ip
echo ${!eth8_ip}
eth8_gw=${host}_eth8_gw
echo ${!eth8_gw}
eth8_mask=${host}_eth8_mask
echo ${!eth8_mask}
eth8_routes=${host}_eth8_routes
echo ${!eth8_routes}
eth8_vipip=${host}_eth8_vipip
echo ${!eth8_vipip}


### FUNCTIONS ###

#etc_hosts ()
#{
#  if [[ "${!eth0_hosts}"  != "" ]]
#  then
#    IFS='#' read -r -a etchosts <<< "${!eth0_hosts}"
#    for iphst in "${etchosts[@]}"
#    do
#      IFS=':' read -r -a element  <<< "${iphst[@]}"
#	  string=""
#	  for ele in "${element[@]}"
#	  do
#	    string=""${string}" $ele"
#	  done
#	  echo $string >> /etc/hosts
#	  #echo ""${element[0]}" "${element[1]}"" >> /etc/hosts
#   done
#  fi  
#}


utc ()
{
  rm -f /etc/localtime
  ln -s /usr/share/zoneinfo/UTC /etc/localtime
  rm -f /etc/sysconfig/clock
  echo 'UTC=true' > /etc/sysconfig/clock
}  

route_file ()
{
routes=$1
gw=$2
int=$3
#table=$4
rm -f /etc/sysconfig/network-scripts/route-${int}
IFS=':' read -r -a route <<< "$routes"
for rt in "${route[@]}"
do
  #if [[ $table != "" ]]
  #then
  #  echo "$rt via $gw table $table" >> /etc/sysconfig/network-scripts/route-${int}
  #else
    echo "$rt via $gw" >> /etc/sysconfig/network-scripts/route-${int}
  #fi
done
}

dummy_int ()
{
ifdown $1
rm -f  /etc/sysconfig/network-scripts/ifcfg-${1}
echo "ONBOOT=no" >> /etc/sysconfig/network-scripts/ifcfg-${1}
}

eth0 ()
{
ifdown eth0
rm -f  /etc/sysconfig/network-scripts/ifcfg-eth0
echo "DEVICE=eth0" > /etc/sysconfig/network-scripts/ifcfg-eth0
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth0
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth0
echo "IPADDR=${!eth0_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth0
echo "NETMASK=${!eth0_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth0
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth0
ifup eth0
intcheck
}

eth1 ()
{
ifdown eth1
rm -f  /etc/sysconfig/network-scripts/ifcfg-eth1
echo "DEVICE=eth1" > /etc/sysconfig/network-scripts/ifcfg-eth1
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth1
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth1
echo "IPADDR=${!eth1_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth1
echo "NETMASK=${!eth1_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth1
if [[ "${!eth1_routes}"  != "" ]]
then
  #if [[ $host == *PD* ]]
  #then
  #  route_file ${!eth1_routes} ${!eth1_gw} eth1 100
  #else
    route_file ${!eth1_routes} ${!eth1_gw} eth1
  #fi
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth1
ifup eth1
}

eth2 ()
{
ifdown eth2
rm -f  /etc/sysconfig/network-scripts/ifcfg-eth2
echo "DEVICE=eth2" > /etc/sysconfig/network-scripts/ifcfg-eth2
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth2
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth2
echo "IPADDR=${!eth2_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth2
echo "NETMASK=${!eth2_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth2
if [[ "${!eth2_routes}"  != "" ]]
then
  route_file ${!eth2_routes} ${!eth2_gw} eth2
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth2
ifup eth2
}

eth3 ()
{
ifdown eth3
rm -f  /etc/sysconfig/network-scripts/ifcfg-eth3
echo "DEVICE=eth3" > /etc/sysconfig/network-scripts/ifcfg-eth3
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth3
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth3
echo "IPADDR=${!eth3_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth3
echo "NETMASK=${!eth3_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth3
if [[ "${!eth3_routes}"  != "" ]]
then
  route_file ${!eth3_routes} ${!eth3_gw} eth3
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth3
ifup eth3
}

eth4 ()
{
ifdown eth4
rm -f  /etc/sysconfig/network-scripts/ifcfg-eth4
echo "DEVICE=eth4" > /etc/sysconfig/network-scripts/ifcfg-eth4
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth4
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth4
echo "IPADDR=${!eth4_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth4
echo "NETMASK=${!eth4_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth4
if [[ "${!eth4_routes}"  != "" ]]
then
  route_file ${!eth4_routes} ${!eth4_gw} eth4
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth4
ifup eth4
}

eth5 ()
{
ifdown eth5
rm -f /etc/sysconfig/network-scripts/ifcfg-eth5
echo "DEVICE=eth5" > /etc/sysconfig/network-scripts/ifcfg-eth5
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth5
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth5
echo "IPADDR=${!eth5_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth5
echo "NETMASK=${!eth5_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth5
if [[ "${!eth5_routes}"  != "" ]]
then
  route_file ${!eth5_routes} ${!eth5_gw} eth5
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth5
ifup eth5
}

eth6 ()
{
ifdown eth6
rm -f /etc/sysconfig/network-scripts/ifcfg-eth6
echo "DEVICE=eth6" > /etc/sysconfig/network-scripts/ifcfg-eth6
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth6
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth6
echo "IPADDR=${!eth6_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth6
echo "NETMASK=${!eth6_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth6
if [[ "${!eth6_routes}"  != "" ]]
then
  route_file ${!eth6_routes} ${!eth6_gw} eth6
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth6
ifup eth6
}

eth7 ()
{
ifdown eth7
rm -f /etc/sysconfig/network-scripts/ifcfg-eth7
echo "DEVICE=eth7" > /etc/sysconfig/network-scripts/ifcfg-eth7
echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth7
echo "NM_CONTROLLED=no" >> /etc/sysconfig/network-scripts/ifcfg-eth7
echo "IPADDR=${!eth7_ip}" >> /etc/sysconfig/network-scripts/ifcfg-eth7
echo "NETMASK=${!eth7_mask}" >> /etc/sysconfig/network-scripts/ifcfg-eth7
if [[ "${!eth7_routes}"  != "" ]]
then
  route_file ${!eth7_routes} ${!eth7_gw} eth7
fi
chmod 0644 /etc/sysconfig/network-scripts/ifcfg-eth7
ifup eth7
}

etc ()
{
  hostname $host
  if [[ $host != *AR* ]]
  then
    echo $installer installer >> /etc/hosts
  else
    echo ${!eth0_ip} installer arbiter >> /etc/hosts
  fi
  echo HOSTNAME=$host >> /etc/sysconfig/network
  echo 127.0.0.1 $host  >> /etc/hosts
}

launchdir ()
{
  mkdir -p /var/lib/cloud/instance/payload/launch-params
  chmod 0664 /var/lib/cloud/instance/payload/launch-params
}

defaultoam ()
{
   echo "GATEWAY=$default_oam_gw" >> /etc/sysconfig/network
}

defaultrepl ()
{
  echo "GATEWAY=$default_repl_gw" >> /etc/sysconfig/network
}

defaultinet ()
{
  echo "GATEWAY=$default_inet_gw" >> /etc/sysconfig/network
}

oamtable ()
{
  p=`ipcalc -p ${!eth1_ip} ${!eth1_mask} | cut -d"=" -f2`
  n=`ipcalc -n ${!eth1_ip} ${!eth1_mask} | cut -d"=" -f2`
  
  echo 100 oam >> /etc/iproute2/rt_tables
  sleep 3
  
  echo "default via $default_oam_gw dev eth1 table 100" >> /etc/sysconfig/network-scripts/route-eth1
  echo "from ${!eth1_ip}/32 table 100" >> /etc/sysconfig/network-scripts/rule-eth1
  echo "to ${!eth1_ip}/32 table 100" >> /etc/sysconfig/network-scripts/rule-eth1
  echo "from ${!eth1_vipip}/32 table 100" >> /etc/sysconfig/network-scripts/rule-eth1
  echo "to ${!eth1_vipip}/32 table 100" >> /etc/sysconfig/network-scripts/rule-eth1
}


debug ()
{
  route
  ifconfig
  cat /etc/sysconfig/network-scripts/ifcfg-eth*
}

intcheck ()
{
x=`ethtool eth0 | grep yes| cut -d: -f2`
if [[ "$x" != " yes" ]]
then
  ifup eth0
  echo "INTERFACE DOWN OR NOT FOUND"
else
  echo "ETH0 FOUND UP"
fi
}

#   VM FUNCTIONS
cm ()
{
  launchdir

  # MODIFY INTERFACES
  eth0
  eth1
  defaultoam
  etc
  service network restart
  
  # WAIT FOR VOLUMES TO PRESENT THEMSELVES
  vdb=/dev/vdb
  #vdc=/dev/vdc
  
  while [[ ! -e $vdb ]]
  do
    sleep 10
    echo "Waiting for ISO device"
  done
  
  # MOUNT INSTALL MEDIA
  mkdir /mnt/iso
  mount -t iso9660 -o auto,ro /dev/vdb /mnt/iso
  # mkdir /var/backup
  # mount -t ext4 /dev/vdc /var/backup
  
  utc
  
  ## CREATE Autoinstall file
  echo '#!/bin/bash' > /root/.autoinstall.sh
  echo 'if [[ -d /mnt/iso ]] && [[ -f /mnt/iso/install.sh ]]' >> /root/.autoinstall.sh
  echo 'then' >> /root/.autoinstall.sh
  echo '/mnt/iso/install.sh << EOF' >> /root/.autoinstall.sh
  echo 'mog' >> /root/.autoinstall.sh
  echo 'y' >> /root/.autoinstall.sh
  echo '1' >> /root/.autoinstall.sh
  echo "$vipnets" >> /root/.autoinstall.sh
  echo 'EOF' >> /root/.autoinstall.sh
  echo 'fi' >> /root/.autoinstall.sh
  chmod 0755 /root/.autoinstall.sh

  /root/.autoinstall.sh
  
  # Install Linux Utilities on CM
  yum -y install man
  yum -y install telnet
  yum -y install traceroute
  yum -y install tcpdump

}

pd ()
{
  node=${host: -2}
  echo "NODE_TYPE=lb${node}" >> /etc/broadhop.profile
  launchdir
  utc
  # MODIFY INTERFACES
  eth0
  eth1
  eth2
  eth3
  dummy_int eth4
  eth5
  eth6
  eth7
  defaultinet
  oamtable
  etc
  service network restart
  debug
}

oam ()
{
  node=${host: -2}
  echo "NODE_TYPE=pcrfclient${node}" >> /etc/broadhop.profile
  launchdir
  utc
  # MODIFY INTERFACES
  eth0
  eth1
  eth2
  defaultoam
  etc
  service network restart
  debug
}

sm ()
{
  node=${host: -2}
  echo "NODE_TYPE=sessionmgr${node}" >> /etc/broadhop.profile
  launchdir
  utc
  # MODIFY INTERFACES
  eth0
  dummy_int eth1
  eth2
  defaultrepl
  etc
  service network restart
  debug
}

ps ()
{
  node=${host: -2}
  echo "NODE_TYPE=qns${node}" >> /etc/broadhop.profile
  launchdir
  utc
  # MODIFY INTERFACES
  eth0
  dummy_int eth1
  eth2
  defaultrepl
  etc
  service network restart
  debug
}

ar ()
{
  node=${host:-2}
  echo "NODE_TYPE=arbiter" >> /etc/broadhop.profile
  launchdir
  utc
  
  # MODIFY INTERFACES
  eth0
  eth1
  eth2
  defaultoam
  etc
  service network restart
  #etc_hosts

  # WAIT FOR DEVICE 
  vdb=/dev/vdb 
  while [[ ! -e $vdb ]]
  do
    sleep 10
    echo "Waiting for ISO device"
  done  
  
  
  mkdir /mnt/iso
  mount -t iso9660 -o auto,ro /dev/vdb /mnt/iso  

  ## CREATE Firewall file
  echo '#!/bin/bash' >> /root/.enable_firewall.sh
  echo 'mkdir -p /etc/facter/facts.d/' >> /root/.enable_firewall.sh
  echo 'echo firewall_disabled=1 >> /etc/facter/facts.d/qps_firewall.txt' >> /root/.enable_firewall.sh
  chmod 0755 /root/.enable_firewall.sh
     
  
  ## CREATE Autoinstall file
  echo '#!/bin/bash' > /root/.autoinstall.sh
  echo 'if [[ -d /mnt/iso ]] && [[ -f /mnt/iso/install.sh ]]' >> /root/.autoinstall.sh
  echo 'then' >> /root/.autoinstall.sh
  echo '/mnt/iso/install.sh << EOF' >> /root/.autoinstall.sh
  echo 'y' >> /root/.autoinstall.sh
  echo '1' >> /root/.autoinstall.sh
  echo 'EOF' >> /root/.autoinstall.sh
  echo 'fi' >> /root/.autoinstall.sh
  echo '/root/.enable_firewall.sh' >> /root/.autoinstall.sh
    
  chmod 0755 /root/.autoinstall.sh

  /root/.autoinstall.sh

}

###################
###    MAIN     ###
###################

###  IF CLUMAN  ###
if [[ $host == *CM* ]]
then
  cm

###  IF POLICY DIRECTOR  ###
elif [[ $host == *PD* ]]
then
  pd

###  IF OAM  ###
elif [[ $host == *OAM* ]]
then
  oam

###  IF SESSION MANAGER  ###
elif [[ $host == *SM* ]]
then
  sm

###  IF POLICY SERVER  ###
elif [[ $host == *PS* ]]
then
  ps
  
###  IF ARBITER ###
#  NEED ATT *MAM*
elif [[ $host == *MAM* ]]
then
  ar
fi